# Generated by Django 5.0.1 on 2024-02-10 08:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('request_filters', '0003_filteredrequest'),
    ]

    operations = [
        migrations.AddField(
            model_name='filteredrequest',
            name='response_info',
            field=models.JSONField(blank=True, default=dict, help_text='Extra information about the response after the request was filtered.', verbose_name='Response Info'),
        ),
    ]
