# Generated by Django 5.0.1 on 2024-02-10 08:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('request_filters', '0002_alter_filter_options_alter_filter_filter_type_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='FilteredRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('_filter', models.JSONField(blank=True, default=dict, help_text='The filter that was triggered.', verbose_name='Filter')),
                ('request_path', models.CharField(help_text='The path of the request that was filtered.', max_length=255, verbose_name='Request Path')),
                ('request_method', models.CharField(help_text='The method of the request that was filtered.', max_length=10, verbose_name='Request Method')),
                ('request_ip', models.GenericIPAddressField(help_text='The IP of the request that was filtered.', verbose_name='Request IP')),
                ('request_language', models.CharField(help_text='The language of the request that was filtered.', max_length=10, verbose_name='Request Language')),
                ('request_is_secure', models.BooleanField(help_text='Whether the request was secure.', verbose_name='Request Is Secure')),
                ('request_info', models.JSONField(blank=True, default=dict, help_text='Extra information about the request that was filtered.', verbose_name='Request Info')),
                ('gis_data', models.JSONField(blank=True, default=dict, help_text='The GIS data of the request that was filtered.', verbose_name='GIS Data')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created At')),
            ],
            options={
                'verbose_name': 'Filtered Request',
                'verbose_name_plural': 'Filtered Requests',
                'ordering': ['-created_at'],
            },
        ),
    ]
