# Generated by Django 5.0.1 on 2024-02-10 06:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('request_filters', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='filter',
            options={'ordering': ['sort_order'], 'verbose_name': 'Filter', 'verbose_name_plural': 'Filters'},
        ),
        migrations.AlterField(
            model_name='filter',
            name='filter_type',
            field=models.CharField(choices=[('IP', 'IP'), ('USER_AGENT', 'User Agent'), ('PATH', 'Path'), ('QUERY_STRING', 'Query String'), ('REFERER', 'Referer'), ('COUNTRY', 'Country'), ('METHOD', 'Method')], default='IP', max_length=20, verbose_name='Filter Type'),
        ),
        migrations.AlterField(
            model_name='filter',
            name='method',
            field=models.CharField(choices=[('ABSOLUTE', 'Absolute'), ('WILDCARD', 'Wildcard'), ('REGEX', 'Regex'), ('IN', 'In')], default='ABSOLUTE', max_length=20, verbose_name='Method'),
        ),
    ]
